/******************************************************************************
 *  Hederdatei ChkArch.C
 *  Deklaration der Funktion ChkAuft() fr das Programm GalAsm, den Logic-
 *  Compiler / -Assembler zur Erstellung einer JEDEC-Datei
 *  fr die GAL-Programmierung.
 *
 *  Version               1.0
 *  Erstellt am           13.08.1990
 *  Letzte nderung am    10.12.1990
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien: GalAsm.H      Makros und Def. der Var.'typen
 *                           GalGlVar.H    Deklaration der globalen Variablen
 *                           HilfFktn.H
 *
 *  Bentigte C-Module:      GalGlVar.C    Definition der globalen Variablen
 *                           HilfFktn.C
 *
 *****************************************************************************/


/*--------------------------------------------------------------------------
 |  Funktion ChkArch()
 |  berprft die Vertrglichkeit zwischen Architekturkontrollbits, den
 |  Rckfhrungen der Ausgangssingale, der Tri-State-Freigabebedingungen
 |  sowie Ausgangsregister der progammierten Ein- und Ausgnge, sowie den
 |  Signalen Clk (Pin 1) und /OE (Pin 11 bzw. 13) und den Pins 14 und 23 bei
 |  Gals des Typs 20V8, soweit sie in der Logik-Matrix bentigt werden.
 |  Fehlercodes:
 */
#define  CHKARCH_ERR  30000L /* Fehlernummer der Fkt. ChkArch()              */
#define OE_CLK_N_EING   300L /* OE, Clk nicht Eingang                        */
#define OLMC_N_RCKK     400L /* i.OLMC: Nicht rckfhrbar                    */
#define KEIN_TS         500L /* i.OLMC: Kein Tri-State mglich               */
#define KEIN_REG        600L /* i.OLMC: Kein registered output mglich       */
#define KEINE_PTF       700L /* i.OLMC: Keine Produkttermfreigabe mglich    */
#define KEINE_OE_FREIG  800L /* i.OLMC: Kein OE-TS mglich                   */
#define KEIN_TSPT_FREI  900L /* i.OLMC: Kein Produktterm fr TS-Freigabe frei*/
#define KEIN_AUSG      1000L /* i.OLMC: Kein Ausgang                         */
#define KEIN_EING      1100L /* i.OLMC: Kein Eingang mglich                 */
 /*
 |  Fr i. wird ein Wert von 0 bis 7 auf die Fehlernummer aufaddiert.
 |  0 entspricht OLMC Nr. 8 (beim OE-Pin) 7 entspricht OLMC Nr. 1 (bei Vcc).
 *--------------------------------------------------------------------------*/

extern long ChkArch(void);
