/******************************************************************************
 *  Headerdatei GALASM.H
 *
 *  Definition der Makros (Konstanten) und Definition der Variablentypen.
 *  Die globalen Variablen werden in der Datei GalGlVar.H deklariert und in
 *  GalGlVar.C definiert.
 *
 *  Fr GALASM.C, dem Logic-Compiler / -Assembler zur Erstellung einer
 *  JEDEC-Datei fr die GAL-Programmierung.
 *
 *  Version               1.1
 *  Erstellt am:	05.08.1990
 *  nderungen:		22.12.1990
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *****************************************************************************/



#define CR     13
#define LF     10

#define STX     2       /* Startzeichen fr JEDEC-Datei */
#define ETX     3       /* Abschluzeichen fr JEDEC-Datei */

#define KEIN_FEHLER     0L       /* globale Fehlercodes */
#define LESEFEHLER      100L
#define SCHREIBFEHLER   200L


#define WARN_ZEILE        8      /* Bildschirmzeile fr Warnun-Meldungen der
                                    Unterprogramme an den/die Benutzer/in */
#define MSG_ZEILE        10      /* Bildschirmzeile fr Mitteilungen der
                                    Unterprogramme an den/die Benutzer/in */
#define END_ZEILE        21


#define ALLES_BRENNEN   100         /* Flag, zeigt an ,da in der Zeile */
                                    /* alle Verbindungen herzustellen sind */
#define NICHTS_BRENNEN -100         /* Flag, zeigt an ,da in der Zeile */
                                    /* alle Verbindungen offen zu lassen sind*/

#define  GAL16V8NR      1       /* Nummer der Gal-Type fr die ... */
#define  GAL20V8NR      2       /* Variable GalTyp */
#define  GAL16V8NAME    "GAL16V8"
#define  GAL20V8NAME    "GAL20V8"

#define  ANZ_PIN_16V8      20
#define  LETZT_E_PIN_16V8  11    /* Nummer des letzten Eing'pins vor den IO */
#define  GND_PIN_16V8      10    /* Nummer des Masse-Pins */
#define  VCC_PIN_16V8      20
#define  OE_PIN_16V8       11
#define  CLK_PIN_16V8       1
#define  SP_OLMC0_16V8     30   /* Pos. Spalte der 0. OLMC */
#define  SP_OLMC7_16V8      2   /* Pos. Spalte der 7. OLMC */




#define  ANZ_PIN_20V8      24
#define  LETZT_E_PIN_20V8  14    /* Nummer des letzten Eing'Pins vor den IO */
#define  GND_PIN_20V8      12    /* Nummer des Masse-Pins */
#define  VCC_PIN_20V8      24
#define  OE_PIN_20V8       13
#define  CLK_PIN_20V8       1
#define  SP_OLMC0_20V8     34   /* Pos. Spalte der 0. OLMC */
#define  SP_OLMC7_20V8      6   /* Pos. Spalte der 7. OLMC */

#define  MAX_PIN        24      /* GAL 20V8 hat 24 Pins */
#define  MAX_ANZ_SPALTEN  20    /* max 20 Spalten in Logikmatrix */

#define  EINGAENGE      14      /* max. Anzahl festverdrahteter Eingnge */
#define  AUSGAENGE      8       /* max. Anzahl programmierbarer Ausgnge */
#define  ANZ_PROD       8       /* 8 Produktterme pro Ausgang */

#define GAL16SPALTEN    16      /* Logikmatrix des GALs 16V8 hat 16 Spalten */
#define GAL20SPALTEN    20      /* Logikmatrix des GALs 20V8 hat 20 spalten */

#define GND_NAME        "Gnd"
#define VCC_NAME        "Vcc"

#define ANZ_ARCH        3    /* Syn, AC0 und AC1(n) = 3 Kontrollbits/-bytes */


#define LOG_ZELLE0      0       /* erste Zelle der Logik-Matrix */

#define XOR_0_G16       2048    /* erste Zelle der XOR-Bits GAL 16V8 */
#define XOR_0_G20       2560    /* erste Zelle der XOR-Bits GAL 20V8 */

#define SIG_0_G16       2056    /* erste Zelle der Signatur-Bits GAL 16V8 */
#define SIG_0_G20       2568    /* erste Zelle der signatur-Bits GAL 20V8 */

#define AC1_0_G16       2120    /* erste Zelle der AC1-Bits GAL 16V8 */
#define AC1_0_G20       2632    /* erste Zelle der AC1-Bits GAL 20V8 */

#define PTF_0_G16       2128    /* erste Zelle der Produkttermfreigabe 16V8 */
#define PTF_0_G20       2640    /* erste Zelle der Produkttermfreigabe 20V8 */

#define SYN_G16         2192    /* Syn-Bit GAL 16V8 */
#define SYN_G20         2704    /* Syn-Bit GAL 20V8 */

#define AC0_G16         2193    /* AC0-Bit GAL 16V8 */
#define AC0_G20         2705    /* AC0-Bit GAL 20V8 */

#define NOTCONNECTED    "NC"    /* Kennzeichnung fr nicht genutzte Pins */

#define PAR_KENNG       '-'     /* Kennzeichnet eine Parameter f. Prg.'aufruf*/
#define WARTEN          'W'     /* Parameter: Warten nach Programmende */
#define FIX_MAXON	'F'	/* Parameter: BugFix fuer MAxon Galprog. */

#define DEFAULT_EXT     "eqn"   /* Defaultextension der Eingabedatei, */
                                /* wird an den Eingabenamen angehngt, */
                                /* falls Datei, mit dem Namen, wie er bei */
                                /* Programmaufruf angegeben wurde, nicht */
                                /* geffnet werden konnte */
#define ZIEL_EXT        "jed"   /* Extension der zu erzeugenden JEDEC-Datei */
#define PROT_EXT        "doc"   /* Extension der Protokolldatei */

#define GALASM_ENV_NAM   "GALASM" /* Name der Environmentvariablen fr das */
                                  /* Programm GalAsm */

#define PATH_ENV_NAM     "PATH"     /* Name der Environmentvariablen des */
                                    /* Betriebssytems f. Suchpfad */

#define ERR_NUM_KENNG   '#'     /* Kennzeichnung eine Fehlernummer in der
                                   Fehlertextdatei */
#define ERR_KOMMENTAR   '%'     /* Kennzeichnung einer Kommentarzeile in der
                                   Fehlertextdatei */
#define ANZ_ERR_ZEILEN  10      /* max. 10 Textzeilen pro Fehlernummer */

#define EING_KOMMENTAR   '#'     /* Kennzeichnung einer Kommentarzeile in der
                                    Eingabedatei */
                                    
#define EING_NEGATION   '/'     /* Kennzeichnung einer Negation in der
                                    Eingabedatei */                                    
                                    

#define ZEILENLAENGE    260     /* 256 Zeichen + CR + LF + NULL-Byte =  */
                                /* Zeilenlnge der zu lesenden Datei =  */
                                /* Pufferlnge */
#define SIGNIF_STELLEN    8     /* Anzahl der signifikanten Stellen v. Namen */

/*--------------------------------------------------
 | Definition der Elemente fr die Listen der Ein- und Ausgnge
 *------------------------------------------------------------------*/
struct EingStructTyp { char PinNummer;  /* Nummer der Pins */
                       char LowAktiv;   /* 0: pos oder 1: neg Logik */
                       char SigName[SIGNIF_STELLEN+1];
                                        /* 8 signifikante Stellen d. Namens */
                       char Genutzt;    /* Flag, zeigt an, da Eing. benutzt */
                     };

struct AusgStructTyp { char PinNummer;  /* Nummer des Pins */
                       char LowAktiv;   /* 0: pos oder 1: neg Logik */
                       char Negieren;   /* 1: Ausgang negieren */
                       char SigName[SIGNIF_STELLEN+1];
                                        /* 8 signifikante Stellen d. Namens */
                       char OEFreig;    /* Singal ist TS, Freig. durch /OE */
                       char ProdFreig;  /* Signal ist TS, Freig. d. Produkt */
                       char Hochohmig;  /* Signal ist TS, immer hochohmig */
                       char Niederohmig;/* Signal ist TS, immer niederohmig */
                       char Rueckkop;   /* eigenen Ausgang zurckkopplen
                                           da in Logikmatrix bentigt       */
                       char Ausg;       /* Signal ist Ausgang */
                       char Reg;        /* Signal wird gelatcht */
                       char Genutzt;    /* Flag, zeigt an, da Ausg. benutzt */
                     };


#define MAX_PATH_LEN  130       /* Max. Lnge fr Pfade */


extern char *VersionNr;
extern char *ErstDatum;

extern int FixMaxon; 

#define UES_LEN    8   /* Lnge der UES in Bytes */


/*=====================
 |   Anpassung div. Bibliotheksfunktionen
 *==============================================*/
#define  stccpy(a,b,c)       	strncpy(a,b,c)
#define  stpchr(a,b)       	strchr(a,b)
#define  strnicmp(a,b,c)	strncasecmp(a,b,c)
#define  stricmp(a,b)		strcasecmp(a,b)

#define gotoxy(a,b)
#define end_line()	
#define cls()	
#define cur_off()	
#define end_screen()