/******************************************************************************
 *  Datei GalGlVar.C
 *  Enthlt die Definitionen aller globalen Variablen fr das Programm
 *  GalAsm, den Logic-Compiler / -Assembler zur Erstellung einer JEDEC-Datei
 *  fr die GAL-Programmierung.
 *
 *  Version               1.1
 *  Erstellt am           11.08.1990
 *  Letzte nderung am    22.12.1990
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *****************************************************************************/

#include <stdio.h>

#include "galasm.h"             /* Einbinden der Makros */


/* Globale Variablen */

int GalTyp = 0;           /* enthlt den Typ des zu programmierenden GALs */
int GndPin = 0;           /* Nummer des Masse-Pins */
int VccPin = 0;           /* Nummer des Versorgungsspannungs-Pins */
int OEPin  = 0;           /* Nummer des Output-Enable-Pins */
int ClkPin = 0;           /* Nummer des Takt-Pins */
int AnzPins= 0;           /* Anzahl Pins an diesem Baustein */
int LetztEingPin = 0;     /* ab diesem Pin folgen die 8 IOs */
int SpOLMC0 = 0;          /* Pos Spalte der OLMC Nr 0 */
int SpOLMC7 = 0;          /* Pos Spalte der OLMC Nr 0 */
int AnzSpalten = 0;       /* Anzahl der Spalten in Logikmatrix */
int AdrXORBits = 0;       /* Adresse des ersten XOR-Bits in JEDEC-Datei */
int AdrSigBits = 0;       /* Adresse des ersten Signatur-Bits in JEDEC-Datei */
int AdrAC1Bits = 0;       /* Adresse des ersten AC1-Bits in JEDEC-Datei */
int AdrPTFBits = 0;       /* Adresse des ersten PTS-Bits in JEDEC-Datei */
int AdrSynBit  = 0;       /* Adresse des Syn-Bits in JEDEC-Datei */
int AdrAC0Bit  = 0;       /* Adresse des AC0-Bits in JEDEC-Datei */


int Warten = 0;           /* Globales Flag, nach Fehlermeldung warten */


char SynBit = -1;          /* Fr die Programmierung der OLMCs */
char AC0Bit = -1;          /*        -     "    -              */
char AC1Bits[AUSGAENGE] = { -1,-1,-1,-1,-1,-1,-1,-1 };

int ClkinMatrix = 0;       /* Flag, zeigt an, da Clockeingang Pin 1 auf Logik-
                              matrix gefhrt werden mu */
int OEinMatrix = 0;        /* Flag, zeigt an, da das OE Signal, Pin 11
                              (GAL 16V8) bzw Pin 13 (GAL 20V8) auf die Logik-
                              Matrix gefhrt werden mu. */

int P14inMatrix = 0;       /* Flag, zeigt an, da bei GAL20V8 Eingang Pin 14
                              auf Logikmatrix gefhrt werden mu */
int P23inMatrix = 0;       /* Flag, zeigt an, da bei GAL20V8 Pin 23
                              auf die Logikmatrix gefhrt werden mu. */

int QuellZeilNr = 0;       /* Zhler fr die aktuelle Zeile der Quelldatei */


char OEMatSp  = 0;   /* Nummer der zugeh. Matrixspalte falls Eing. bentigt */
char ClkMatSp = 0;   /*                     -     "     -                   */
char P14MatSp = 0;   /*                -     "     -      nur fr Gal 20V8  */
char P23MatSp = 0;   /*                -     "     -      nur fr Gal 20V8  */

struct EingStructTyp Eingaenge[EINGAENGE];
struct AusgStructTyp Ausgaenge[AUSGAENGE];

/*-------------------------------
 | Feld, in welches der Parser bei der Analyse der Produktterme fr jeden
 | Ausgang die Verknpfung der Signale eintrgt.
 | Zugeordnet werden jedem Ausgang soviele Felder von Bytes, wie
 | dieser Ausgang Produkte in seinem Produktterm besitzt.
 | Die Felder sind solang, wie die Logikmatrix breit (max. Anzahl von Spalten).
 | In das Feld werden die Pinnummern der bentigten Signale nach folgendem
 | Schema eingetragen:  0 => kein Signal (Fuse nicht gebrannt)
 |                     +x => Verbindung zu Signal von Pin x
 |                     -x => Verbingung zu invertiertem Signal von Pin x
 *------------------------------------------------------------------------*/
char LogikMatrix[AUSGAENGE][ANZ_PROD][MAX_ANZ_SPALTEN];

/* Liste der Spaltennummern in Logikmatrix fr jeden Pin, norm. Signale */
int PosPinInSp[MAX_PIN];
/* Liste der Spaltennummern in Logikmatrix fr jeden Pin, inv. Signale */
int InvPinInSp[MAX_PIN];


/* Fr jeden Ausgang acht Produkttermfreigabebits */
char ProdTermFreig[AUSGAENGE][ANZ_PROD];

FILE *QuellFP;          /* Filepointer Quelldatei */
FILE *ZielFP;           /* Filepointer Zieldatei */
FILE *ProtFP;           /* Filepointer Protokolldatei */
FILE *ErrFP;            /* Filepointer der Fehlertextdatei */

char *Buffer;           /* Puffer fr Stringoperationen */

char *ErrDatNam = "galasm.err";   /* Name der Fehlertextdatei */


char UserElSig[UES_LEN+1];    /* String, nimmt die UES auf */
