/******************************************************************************
 *  Datei GetGal.C
 *  Enthlt die Funktion GetGal() fr GalAsm, den Logic-Compiler /
 *  -Assembler zur Erstellung einer JEDEC-Datei fr die GAL-Programmierung.
 *
 *  Version               1.2
 *  Erstellt am           11.08.1990
 *  Letzte nderung am    3.1.1991
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien: GalAsm.H      Makros und Typendef. der Variablen
 *                           GalGlVar.H    Deklaration der globalen Variablen
 *                           HilfFktn.H    Hilfsfunktionen
 *
 *  Bentigte C-Module:      GalGlVar.C
 *                           HilfFktn.C
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "galasm.h"
#include "galglvar.h"
#include "hilffktn.h"

/*------------------------------
 | Fehlercodes
 *------------------------------*/

/*--------------------------------------------------------------------------
 | Funktion bestimmt den GAL-Typ, trgt diesen in das Flag GalTyp ein und
 | analysiert die Liste der symbolischen Singalnamen. Namen und zugeh.
 | Pinnummern werden in die beiden Felder Eingaenge[] und Ausgaenge[] ein-
 | getragen. Bei einem Fehler wird unter Rckgabe eines Fehlercodes abge-
 | brochen.   Fehlercode:
 */
#define GETGAL_ERR    10000L /* Basis-Fehlercode von GetGal()                */
                             /* 1 bis 24: Pinnummer, bei der Fehler auftrat. */
#define GAL_UNBEK       300L /* GAL-Typ unbekannt              }             */
#define ZUWENIG_NAMEN   400L /* Zuwenig Signalnamen            } Fehlernrn.  */
#define ZUVIELE_NAMEN   500L /* Zuviele Signalnamen            } werden ggf. */
#define GND_PIN_FALSCH  600L /* Gnd an falschem Pin            } auf die Pin-*/
#define VCC_PIN_FALSCH  700L /* Vcc an falschem Pin            } nummer add. */
#define NAME_N_EIND     800L /* Signalname nicht eindeutig     }             */
#define NAME_UNBEK      900L /* Signalname ungltig            }             */
#define GLEICH_ERW     1000L /* Erwarte Gleichzeichen (=)      }             */
#define BINAER_ERW     1100L /* Erwarte 0 oder 1               }             */
#define TRENN_ERW      1200L /* Erwarte Trennungszeichen (,;=) }             */
/*---------------------------------------------------------------------------*/

long GetGal()
{
  char *Ptr;            /* Hilfszeiger */
  char *NeuPtr;
  int PinNummer;        /* Laufvariable */
  char SigName[SIGNIF_STELLEN+1];      /* Speichert Signalnamen */
  char LowAktiv;        /* Flag fr negative Logik */
  register int i,k;     /* Laufvariable */

  gotoxy(0,MSG_ZEILE);
  end_line();
#ifdef GALASM_DEBUG  
  printf("Scannen von GAL-Typ und Signalnamen...\n");
#endif
  if( ! (Ptr = LeseZeile())) return(GETGAL_ERR + LESEFEHLER);
  if (! strnicmp(Ptr,GAL16V8NAME,strlen(GAL16V8NAME))) {
    GalTyp = GAL16V8NR;
  }
  else {
    if(! strnicmp(Ptr,GAL20V8NAME,strlen(GAL20V8NAME))) {
      GalTyp = GAL20V8NR;
    }
    else {
      return(GETGAL_ERR + GAL_UNBEK);   /* GalTyp unbekannt */
    }
  }
  if (! fprintf(ZielFP,"%c\nZiel-PLD: %s\n\n",STX,Ptr)) /* Kennz. f. JEDEC- */
    return(GETGAL_ERR + SCHREIBFEHLER);        /* Datei + GAL-Typ eintragen */
  if (! fprintf(ZielFP,
        "Logik-Compilat erstellt mit GALASM, Ver %s vom %s von V.Bosch\n",
            VersionNr,ErstDatum))
    return(GETGAL_ERR + SCHREIBFEHLER);
  if (! fprintf(ZielFP,"=============================================\
==================================\n\n"))
    return(GETGAL_ERR + SCHREIBFEHLER);
  switch(GalTyp) {

    case GAL16V8NR: { AnzPins = ANZ_PIN_16V8;
                      LetztEingPin = LETZT_E_PIN_16V8; /* ab hier 8 IOs */
                      GndPin     = GND_PIN_16V8;  /* Nummer des Masse-Pins */
                      VccPin     = VCC_PIN_16V8;
                      OEPin      = OE_PIN_16V8;
                      ClkPin     = CLK_PIN_16V8;
                      AnzSpalten = GAL16SPALTEN;
                      AdrXORBits = XOR_0_G16;
                      AdrSigBits = SIG_0_G16;
                      AdrAC1Bits = AC1_0_G16;
                      AdrPTFBits = PTF_0_G16;
                      AdrSynBit  = SYN_G16;
                      AdrAC0Bit  = AC0_G16;
                      SpOLMC0 = SP_OLMC0_16V8;   /* Pos. Spalte der 0. OLMC */
                      SpOLMC7 = SP_OLMC7_16V8;   /* Pos. Spalte der 7. OLMC */
                    } break;
    case GAL20V8NR: { AnzPins = ANZ_PIN_20V8;
                      LetztEingPin = LETZT_E_PIN_20V8;  /* ab hier 8 IOs */
                      GndPin = GND_PIN_20V8;        /* Nummer des Masse-Pins */
                      VccPin = VCC_PIN_20V8;
                      OEPin =  OE_PIN_20V8;
                      ClkPin = CLK_PIN_20V8;
                      AnzSpalten = GAL20SPALTEN;
                      AdrXORBits = XOR_0_G20;
                      AdrSigBits = SIG_0_G20;
                      AdrAC1Bits = AC1_0_G20;
                      AdrPTFBits = PTF_0_G20;
                      AdrSynBit  = SYN_G20;
                      AdrAC0Bit  = AC0_G20;
                      SpOLMC0 = SP_OLMC0_20V8;   /* Pos. Spalte der 0. OLMC */
                      SpOLMC7 = SP_OLMC7_20V8;   /* Pos. Spalte der 7. OLMC */
                    } break;
  } /* switch */
  for(i=0;i<3;i++) {
  if( ! (Ptr = fgets(Buffer,ZEILENLAENGE+2,QuellFP))) /* 3 Kommentarzeilen */
    return(GETGAL_ERR + LESEFEHLER);                  /* lesen und als Kom-*/
  QuellZeilNr++;           /* Zhler akt. Zeile */    /* tarzeilen in die  */
  NeuPtr = Ptr;                                       /* JEDEC-Datei eintr.*/
  while(*NeuPtr) {                                    /* Steuerzeichen f.  */
    if(*NeuPtr == '*') *NeuPtr = 'X';                 /* JEDEC-Datei durch */
    NeuPtr++;                                         /* Buchstabe X       */
  }                                                   /* ersetzen          */
  if (! fprintf(ZielFP,"%s",Ptr))
    return(GETGAL_ERR + SCHREIBFEHLER);
  }
  if (! fprintf(ZielFP,"*F1\n*G0\n")) /* G0*: Securitycell nicht brennen */
                                    /*F1: nicht bentigte Fuses nicht brennen*/
    return(GETGAL_ERR + SCHREIBFEHLER);
  /*---- Architekturkontrollworte lesen, falls angegeben -------*/
  if( ! (Ptr = LeseZeile())) return(GETGAL_ERR + LESEFEHLER);
  i = 0;  /* Zhlvariable inintialisieren */
  do {
    Ptr = LeerzUeberl(Ptr);
    NeuPtr = GetSigName(Ptr,SigName);
    if (! NeuPtr) {
      return(GETGAL_ERR + NAME_UNBEK);  /* Kein gltiger Signalname */
    }
    if(! stricmp(SigName,"Syn")) {
      i++;
      NeuPtr = LeerzUeberl(NeuPtr);
      if(! NeuPtr) return(GETGAL_ERR + LESEFEHLER);
      if(*NeuPtr++ != '=') {
        return(GETGAL_ERR + GLEICH_ERW);    /* Erwarte =Zeichen */
      }
      NeuPtr = LeerzUeberl(NeuPtr);
      if(! NeuPtr) return(GETGAL_ERR + LESEFEHLER);
      if(*NeuPtr == '0') SynBit = 0;
        else if(*NeuPtr == '1') SynBit = 1;
          else {
            return(GETGAL_ERR + BINAER_ERW);       /* Erwarte 0 oder 1 */
          }
      Ptr = ++NeuPtr;
    }
    else {
      if(! stricmp(SigName,"AC0")) {
        i++;
        NeuPtr = LeerzUeberl(NeuPtr);
        if(! NeuPtr) return(GETGAL_ERR + LESEFEHLER);
        if(*NeuPtr++ != '=') {
          return(GETGAL_ERR + GLEICH_ERW);
        }
        NeuPtr = LeerzUeberl(NeuPtr);
        if(! NeuPtr) return(GETGAL_ERR + LESEFEHLER);
        if(*NeuPtr == '0') AC0Bit = 0;
          else if(*NeuPtr == '1') AC0Bit = 1;
            else {
              return(GETGAL_ERR + BINAER_ERW);
            }
        Ptr = ++NeuPtr;
      }
      else {
        if(! stricmp(SigName,"AC1")) {
          i++;
          NeuPtr = LeerzUeberl(NeuPtr);
          if(! NeuPtr) return(GETGAL_ERR + LESEFEHLER);
          if(*NeuPtr == '(') {           /* AC1(n) */
            NeuPtr++;                    /* ( berlesen */
            NeuPtr = LeerzUeberl(NeuPtr);     /* zeigt auf n */
            if(! NeuPtr) return(GETGAL_ERR + LESEFEHLER);
            NeuPtr++;                      /* n berlesen */
            NeuPtr = LeerzUeberl(NeuPtr);     /* zeigt auf ) */
            if(! NeuPtr) return(GETGAL_ERR + LESEFEHLER);
            NeuPtr++;                      /* ) berlesen */
          }
          NeuPtr = LeerzUeberl(NeuPtr);
          if(! NeuPtr) return(GETGAL_ERR + LESEFEHLER);
/*------------- alte Version:  ----     for(k=0;k<AUSGAENGE;k++){---------- */
          for(k=AUSGAENGE-1;k>=0;k--){  /* Pro Ausgang ein AC1(n)-Bit */
            if(! ((*NeuPtr == '=') ||
                  (*NeuPtr == ',') ||
                  (*NeuPtr == EING_KOMMENTAR)    )) {
                     return(GETGAL_ERR + TRENN_ERW); /* Erwarte Trennungs- */
                                               /* zeichen , oder  ; oder =   */
            }
            NeuPtr++;           /* Trennzeichen berlesen */
            NeuPtr = LeerzUeberl(NeuPtr);     /* Leerzeichen berlesen */
            NeuPtr=GetSigName(NeuPtr,SigName);/*folgendes Zeichen herauslsen*/
            if (! NeuPtr) return(GETGAL_ERR + NAME_UNBEK);  /*Name nicht gef.*/
            NeuPtr = LeerzUeberl(NeuPtr);     /* Leerzeichen berlesen */
            if (*SigName == '0') AC1Bits[k] = 0;
            else if (*SigName == '1') AC1Bits[k] = 1;
              else {
                return(GETGAL_ERR + BINAER_ERW);
              }
          }   /* for(k) */
          Ptr = NeuPtr;
        }
        else (i=0);
      }
    }
  } while((i) && (i< ANZ_ARCH));
  PinNummer = 1;
  while(PinNummer <= AnzPins)  {          /* Schleife ber Pins */
    do {
      Ptr = LeerzUeberl(Ptr);
      Ptr = SignalNeg(Ptr,&LowAktiv);
      Ptr = GetSigName(Ptr,SigName);
      if (! Ptr)              /* Singalname nicht gefunden */
        return(GETGAL_ERR + NAME_UNBEK + PinNummer);
      if(stricmp(NOTCONNECTED,SigName)) {    /* Signame <> Notconnetced */
        for(i=0;( (i<EINGAENGE) && Eingaenge[i].PinNummer );i++) {/* Prfen, */
          if( ! stricmp(Eingaenge[i].SigName,SigName))   {       /* ob Sig- */
            return(GETGAL_ERR + PinNummer + NAME_N_EIND);         /* nalname */
          }
        }                                                         /* bereits */
        for(i=0;( (i<AUSGAENGE) && Ausgaenge[i].PinNummer );i++) {/* vorhan- */
          if( ! stricmp(Ausgaenge[i].SigName,SigName))  {         /* den     */
            return(GETGAL_ERR + PinNummer + NAME_N_EIND);  /* ggf. Abbruch   */
          }
        }
      }
      if(PinNummer < GndPin) {  /* Pins sind Eingnge */
        Eingaenge[PinNummer-1].PinNummer = PinNummer;
        Eingaenge[PinNummer-1].LowAktiv = LowAktiv;
        strcpy(Eingaenge[PinNummer-1].SigName,SigName);
        Eingaenge[PinNummer-1].Genutzt = 0;             /* Flag lschen */
      }
      else {   /* 1. else */
        if(PinNummer == GndPin) {
          if(stricmp(GND_NAME,SigName))  {
              return(GETGAL_ERR + PinNummer + GND_PIN_FALSCH);
          }
        }
        else {    /* 2. else */
          if(PinNummer <= LetztEingPin) {  /* Pin ist noch Eingang */
            Eingaenge[PinNummer-2].PinNummer = PinNummer;   /* beachte: der */
            Eingaenge[PinNummer-2].LowAktiv = LowAktiv;     /*Gnd-Pin mu ab-*/
            strcpy(Eingaenge[PinNummer-2].SigName,SigName); /*gezogen werden*/
            Eingaenge[PinNummer-2].Genutzt = 0;             /* Flag lschen */
          }
          else {  /*  3. else */
            if (PinNummer <= (LetztEingPin + AUSGAENGE)) {     /* Pin ist IO */
              Ausgaenge[PinNummer-LetztEingPin-1].PinNummer = PinNummer;
              Ausgaenge[PinNummer-LetztEingPin-1].LowAktiv = LowAktiv;
              Ausgaenge[PinNummer-LetztEingPin-1].Negieren = LowAktiv;
              strcpy(Ausgaenge[PinNummer-LetztEingPin-1].SigName,SigName);
            }
            else {  /* 4. else */
              if(PinNummer < AnzPins)  { /* Pin ist wieder Eingang */
               Eingaenge[PinNummer-10].PinNummer=PinNummer;/* beachte: der   */
               Eingaenge[PinNummer-10].LowAktiv=LowAktiv;  /* Gnd-Pin und die*/
                                       /*  8 IO-Pins mssen abgezogen werden */
               strcpy(Eingaenge[PinNummer-10].SigName,SigName);
               Eingaenge[PinNummer-10].Genutzt = 0;         /* Flag lschen */
              }
              else {  /* 5. else */
                if (PinNummer == VccPin) {
                  if(stricmp(VCC_NAME,SigName))  {
                    return(GETGAL_ERR + PinNummer + VCC_PIN_FALSCH);
                  }
                }
              } /* 5. else */
            }  /* 4. else */
          }  /* 3. else */
        }  /* 2. else */
      } /* 1. else */
      PinNummer++;
      while(*Ptr && isspace(*Ptr)) Ptr++;         /* Leerzeichen berlesen */
    } while ((*Ptr != '\0') && (*Ptr != EING_KOMMENTAR)); /* wenn Zeilenende oder */
                            /* Kommentarzeichen erreicht, neue Zeile lesen */
  }      /* while (PinNummer<AnzPins) */
  gotoxy(0,MSG_ZEILE);
  end_line();
  return(KEIN_FEHLER);
}

