/******************************************************************************
 * Datei HilfFktn.C
 *
 *  Beinhaltet Funktionen, welche von den einzelnen Programmteilen des
 *  Programms bentigt werden. Fr GalAsm, den Logic-Compiler / -Assembler
 *  zur Erstellung einer JEDEC-Datei fr die GAL-Programmierung.
 *
 *  Version              1.2
 *  Erstellt am          11.8.1990
 *  Letzte nderung am   3.1.1991
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien: GalAsm.H      Makros und Def. der Var.'typen
 *                           GalGlVar.H    Deklaration der globalen Variablen
 *
 *  Bentigte C-Module:      GalGlVar.C    Definition der globalen Variablen
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>


#include "galasm.h"
#include "galglvar.h"

char *LeerzUeberl(char *);
char *SignalNeg(char *, char *);
char *LeseZeile(void);
char *GetSigName(char *, char *);
int   GetEingPinNr(char *);
int   GetAusgPinNr(char *);
int   ProdTermFrei(int);

/*-------------------------------------------------------------------
 |  Funktion liest eine Zeile aus der Quelldatei ein und gibt einen Zeiger
 |  auf das erste Zeichen, das kein Leerzeichen ist, zurck.
 |  Die Funktion inkrementiert den Zeilenzhler QuellZeilNr.
 |  CR und LF am Zeilenende werden durch NULL-Bytes ersetzt.
 |  Leer- und reine Kommentarzeilen werden berlesen.
 |  Rckabewert: NULL-Zeiger, falls IO-Fehler (wie z.B. Datei leer).
 *---------------------------------------------------------------------*/
char *LeseZeile()
{
  char *Ptr;
  char *HPtr;    /* Hilfszeiger */

  do {
    if( ! (Ptr = fgets(Buffer,ZEILENLAENGE,QuellFP))) return(NULL);
    QuellZeilNr++;           /* Zhler fr die aktuelle Zeile der Quelldatei */
    HPtr = Ptr;
    while(*HPtr) {     /* CR und LF im String durch NULL-Bytes ersetzen */
      if (*HPtr == CR) {
        *HPtr = '\0';
        break;
      }
      if(*HPtr == LF)  {
        *HPtr = '\0';
        break;
      }
      HPtr++;
    }
    while((*Ptr)    &&
          (isspace(*Ptr) 	||           /* Leerzeichen und */
          (!(isprint(*Ptr)))))      /* Nichtdruckende Zeichen */
             Ptr++;                 /* berlesen */
  } while ((*Ptr == '\0')    ||    /* wenn Zeile leer oder  */
      (*Ptr == EING_KOMMENTAR)  ); /* reine Kommentarzeile, neue Zeile lesen */
  return(Ptr);
}

/*----------------------------------------------------------------------
 | berliest die Leerzeichen, auf die der Zeiger ZeilPtr zeigt.
 | Wenn Zeilenende (NULL-Byte) oder Kommentar (;) erreicht, wird
 | eine neue Zeile gelesen. Rckgabewert: Zeiger auf erstes Zeichen, das
 | kein Leerzeichen mehr ist.
 | Bei Fehler (unerwartetes Dateiende) wird ein NULL-Zeiger zurckgegeben
 *-------------------------------------------------------------------------*/
char *LeerzUeberl(ZeilPtr)
  char *ZeilPtr;
  {
    if( ! ZeilPtr) return(NULL);
    while((*ZeilPtr) &&
          (isspace(*ZeilPtr)	||        /* Leerzeichen und */
          (!(isprint(*ZeilPtr)))))   /* nichtdruckende Zeichen berlesen */
      ZeilPtr++;
    if((! *ZeilPtr)|| (*ZeilPtr == EING_KOMMENTAR)) { /* Zeilenende oder Kommentar */
      if(! (ZeilPtr = LeseZeile()))        /* neue Zeile lesen */
        return(NULL);                      /* Fehler ! */
    }
    return(ZeilPtr);
  }


/*---------------------------------------------------------------------------
 | Funktion SignalNegativ() stellt fest, ob das Signal, auf dessen Namen
 | der Zeiger SigPtr zeigt einen oder mehere Not-Operatoren (/) fhrt.
 | Der erste Not-Operator setzt das Negativflag, das per reverence ber-
 | geben werden mu, alle weiteren toggeln das Flag. Wird kein Not-Operator
 | gefunden, so wird das Naegativflag gelscht.
 | Rckgabewert: Neue Stellung des Zeigers SigPtr. Im Fehlerfall den
 | ursprnglichen Wert von SigPtr.
 *---------------------------------------------------------------------------*/
char *SignalNeg(SigPtr,NegFlag)
  char *SigPtr;
  char *NegFlag;
  {
    if(! *SigPtr) return(SigPtr);
    if(*SigPtr == EING_NEGATION) {
      *NegFlag = 1;
      SigPtr++;
    }
    else *NegFlag = 0;
    while(*SigPtr && (*SigPtr == EING_NEGATION)) {
      *NegFlag = !*NegFlag;
      SigPtr++;
    }
  return(SigPtr);
  }

/*-----------------------------------------------------------------------
 * Liefert 1 zurck, wenn das Zeichen 'Character' ein gltiger Bezeichner
 * fr Signal-Namen ist
 *-------------------------------------------------------------------------*/
int IsNamChar(register char Character)
{
  static char *NamIdChar = "_$%&~@";
  
  if(isalnum(Character)) return(1);
  if(index(NamIdChar, Character) != NULL) return(1);
  return(0);
}

/*-------------------------------------------------------------------------
 | Funktion kopiert aus der Zeichenkette, auf die String zeigt, den ersten
 | Signalnamen in die Zeichenkette, auf die NamPtr zeigt.
 | Bercksichtigt werden nur die acht ersten Stellen.
 | Bei Erreichen des ersten nicht alphanummerischen Zeichens wird der
 | Kopiervorgang beendet und die herauskopierte Zeichenkette mit einem NULL-
 | Byte abgeschlossen.
 | Rckgabe: Neue Position des Stringzeigers String, auf das erste folgende
 | nicht alphanummerische Zeichen.
 *---------------------------------------------------------------------------*/
 char *GetSigName(String,NamBuffer)
 char *String;
 char *NamBuffer;
 {
   int i;

   i=0;
   if(( ! *String) ||                 /* wenn bereits Zeilenende erreicht */
      (! IsNamChar(*String)) )  {       /* oder Zeilenrest Kommentar ist */
         *NamBuffer = '\0';           /* Puffer fr Sig' Namen lschen und */
         return(String);              /* den Zeiger unverndert zurckgeben */
   }


   while( (i<SIGNIF_STELLEN) &&      /* bis zum ersten nicht alphanum- */
          (*String) &&                   /* merischen Zeichen, aber max */
          IsNamChar(*String) ) {           /* SIGNIF_STELLEN Zeichen kopieren */
                             i++;
                             *NamBuffer++ = *String++;
   }
   *NamBuffer = '\0';      /* kopierten String mit 0-Byte abschlieen */
   while( *String && IsNamChar(*String))  /* Zeiger auf nchstes nicht */
      String++;                         /* alphanum. Zeichen */
   return(String);
 }


/*----------------------------------------------------------------------
 |  Funktion GetEingPinNr(SigName)
 |  Liefert die Nummer des Eingangspins an dem das Signal mit dem Namen
 |  SigName anliegt und setzt ggf. die Flags ClkinMatrix (Clockeingang
 |  mu auf Logikmatrix gefhrt werden) und OEinMatrix (OE-Signal mu
 |  mu auf Logikmatrix gefhrt werden) sowie P14inMatrix und P23inMatrix
 |  bei GAL20V8.
 |  Auerdem wird das Flag Genutzt gesetzt, um anzuzeigen, da der Ein-
 |  gang benutzt wird.
 |  Im Fehlerfall Rckgabewert: 0.
 *-----------------------------------------------------------------------*/
int GetEingPinNr(SigName)
  char *SigName;
  {
    int i;
    int Pin;

    for(i=0;((i<EINGAENGE) && (Eingaenge[i].PinNummer));i++) {
      if(! stricmp(SigName,Eingaenge[i].SigName))  {
        if(! stricmp(Eingaenge[i].SigName,NOTCONNECTED))  /* Eing als nc */
          return(0);                                      /* gekennz. */
        Eingaenge[i].Genutzt = 1;               /* Flag setzen */
        Pin = Eingaenge[i].PinNummer;
        if(Pin == OEPin) OEinMatrix = 1;
        if(Pin == ClkPin) ClkinMatrix = 1;
        if((Pin == 14) && (GalTyp == GAL20V8NR))  /* Eingang Pin 14 mu auf */
          P14inMatrix = 1;                     /* Logikmatrix gefhrt werden */
        if((Pin == 23) && (GalTyp == GAL20V8NR))  /* Eingang Pin 23 mu auf */
          P23inMatrix = 1;                     /* Logikmatrix gefhrt werden */
        if(Eingaenge[i].LowAktiv) Pin = -Pin;
        return(Pin);
      }
    }
    return(0);
  }



/*----------------------------------------------------------------------
 |  Funktion GetAusgPinNr(SigName)
 |  Liefert die Nummer des Ausgangspins an dem das Signal mit dem Namen
 |  SigName anliegt. auerdem wird fr den zugeh. Ausgang das Flag
 |  Rueckkop gesetzt.
 |  Im Fehlerfall Rckgabewert 0.
 *-------------------------------------------------------------------------*/
int GetAusgPinNr(SigName)
  char *SigName;
  {
    int i;

    for(i=0;((i<AUSGAENGE) && (Ausgaenge[i].PinNummer));i++) {
      if(! stricmp(SigName,Ausgaenge[i].SigName)) {
        Ausgaenge[i].Genutzt = 1;               /* Flag setzen */
        Ausgaenge[i].Rueckkop = 1;
        return((int)Ausgaenge[i].PinNummer);
      }
    }
    return(0);
  }



/*--------------------------------------------------------------------------
 | Funktion ProdTermFrei() prft ob in dem dem bergebenen Ausgang (0 bis
 | AUSGAENGE-1) noch ein Produktterm frei ist und kopiert ggf. den 0. Pro-
 | duktterm an die freie Stelle. Der 0. Term wird gelscht.
 | Rckgabewert: <0 im Fehlerfall, sonst 0.
 *---------------------------------------------------------------------------*/
int ProdTermFrei(AusgNr)
 int AusgNr;
 {
   register int k;
   int FreiProdTerm;

   FreiProdTerm = -1;
   if((AusgNr<0) || (AusgNr >= AUSGAENGE)) return(-1);
   k = 0;
   do {
     if(! LogikMatrix[AusgNr][k][0]) FreiProdTerm = k;
     k++;
   } while((k < ANZ_PROD) && (FreiProdTerm < 0));
   if(FreiProdTerm < 0) return(-2);          /* nein -> Fehler */
   for(k=0;k<MAX_ANZ_SPALTEN;k++) {         /* 0. Produkteterm in den freien */
     LogikMatrix[AusgNr][FreiProdTerm][k]=  /* Produktterm kopieren*/
     LogikMatrix[AusgNr][0][k];
     LogikMatrix[AusgNr][0][k] = 0;         /* 0. Term lschen */
   }
   return(0);
 }

