/******************************************************************************
 *  Headerdatei Parser.H
 *  Deklariert die Funktion Parser() fr GalAsm, den Logic-Compiler / -
 *  Assembler zur Erstellung einer JEDEC-Datei fr die GAL-Programmierung.
 *
 *  Version               1.0
 *  Erstellt am           10.08.1990
 *  Letzte nderung am    10.12.1990
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien:  GalAsm.H     Konstanten und Def. der Var.'typen
 *                            GalGlVar.H   Deklaration der globalen Variablen
 *                            HilfFktn.H   Deklaration der Hilfsfunktionen
 *
 *  Bentigte C-Module:       GalGlVar.C
 *                            HilfFktn.C
 *
 *****************************************************************************/



/*-------------------------------------------------------------------------
 | Funktion Parser
 | Analysiert die booleschen Gleichungen fr die Ausgnge.
 |  1. Feststellen, ob Tristate-Ausgang
 |  2. Falls 1., dann Freigabebedingung analysieren und entspr. Flags setzen:
 |        a. immer niederohmig IF(Vcc)  bzw. IF(/Gnd)
 |        b. immer hochohmig   IF(/Vcc) bzw. IF(Gnd)
 |        c. Freigabe durch /OE (Signal an Pin 11 bzw. 13, low-aktiv)
 |        d. Freigabe durch Produkt
 |  3. Falls 2.d. Analyse des Produkts:
 |        Zustand ermitteln (normal oder invertiert).
 |        Signalnamen ermitteln.
 |        Pinnummer des Signals ermitteln.
 |        Falls Signal /OE ist (Pin 11 oder 13) Flag OEinMatrix setzen.
 |        Falls Signal Clk ist (Pin 1) Flag ClkinMatrix setzen.
 |        Falls Signal ein Ausgang ist, fr diesen Ausgang das Rckkoppelflag
 |              setzen.
 |        Pinnummer mit Vorzeichen in das Feld eintragen.
 |        Fortsetzen, bis Produkt abgearbeitet.
 |  4. Falls nicht 3., dann Flags Tristate und Niederohmig setzen
 |  5. Name des Ausgangs ermitteln.
 |  6. Lfd. Nummer des Ausgangs anhand der Liste der Ausgnge ermitteln
 |  7. Analyse der Zuordnung: Normal (=) oder Registered (:=) und ggf. Flag
 |        fr Registered Output setzen.
 |  8. Analyse des Produktes, wie unter 3.
 |  9. 8. wiederholen, bis Produktterm abgearbeitet.
 | 10. Nchsten Ausgang bearbeiten, wie unter 1. bis 9.
 | 11. 10. wiederholen, bis alle Ausgnge abgearbeitet sind.
 |
 |  Rckgabewerte: Fehlercode:
 |*/
#define PARSER_ERR        20000L  /* Basis-Fehlercode fr Parser() */
                        /*  100L  Lesefehler,    in Galasm.H definiert */
                        /*  200L  Schreibfehler, in Galasm.H definiert */
#define ZU_VIEL_AUSG        300L  /* Zu viele Ausgnge angegeben */
#define FREIG_TERM          400L  /* Freigabeproduktterm falsch */
#define EING_UNBEK          500L  /* Signalname unbekannt */
#define NICHT_AUSG          600L  /* Signal ist kein Ausgang */
#define CHKFEHLER           700L  /* Kontrolle falsch */
#define ZU_VIEL_PROD        800L  /* mehr Produkte als Matrixspalten */
#define SIG_UNBEK           900L  /* Signalname nicht gefunden */
#define KEIN_FREIG         1000L  /* Freigabebedingung fehlt */
#define AUSG_N_GEFUNDEN    1100L  /* Ausgangssignal nicht gefunden */
#define ZUORDNG_FALSCH     1200L  /* Zuordnug falsch */
#define ZU_VIEL_TERME      1300L  /* mehr Produkte als Matrixspalten */
 /*
 *--------------------------------------------------------------------------*/
extern long Parser(void);

