/******************************************************************************
 * Datei PinInSp2.C
 *
 *  Beinhaltet die Funktion PinInSpalte(), fr das Programm GalAsm, den Logic-
 *  Compiler / -Assembler zur Erstellung einer JEDEC-Datei fr die
 *  GAL-Programmierung.
 *
 *  Version              1.3
 *  Erstellt am          13.8.1990
 *  Letzte nderung am   28.7.1991
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien: GalAsm.H      Makros und Def. der Var.'typen
 *                           GalGlVar.H    Deklaration der globalen Variablen
 *
 *  Bentigte C-Module:      GalGlVar.C    Definition der globalen Variablen
 *
 *****************************************************************************/

#include <stdio.h>

#include "galasm.h"
#include "galglvar.h"

/*-------------------------------------------------------------------
 | Funktion PinInSpalte(), erzeugt die
 | Zuordnungstabelle Pin-Nummer  <-> Spalte der Logikmatrix fr pos Signal
 |                                   Spalte der Logikmatrix fr inv Signal
 | Fehlercodes:
 */
#define PININSP_ERR   40000L     /* Basis-Fehlernummer */
 /*
 *---------------------------------------------------------------------*/
long PinInSpalte()
{
register int i;


  gotoxy(0,MSG_ZEILE);
  end_line();
#ifdef GALASM_DEBUG
  printf("Zuordnung: Pinnummer - Spalten der Logikmatrix...\n");
#endif
  if(SynBit) {
    PosPinInSp[ClkPin-1] = 2;      /* 0 = Pin 1: Clk-Pin */
    InvPinInSp[ClkPin-1] = 3;
  }
  else {
    PosPinInSp[ClkPin-1] = -1;
    InvPinInSp[ClkPin-1] = -1;
  }
  for(i=1;i<GndPin-1;i++) {    /* 1 = Pin 2 bis 8 = Pin 9 (16V8)  */
    PosPinInSp[i] = (i-1)*4;        /* bzw bis 10  = Pin 11 (20V8) */
    InvPinInSp[i] = ((i-1)*4)+1;
  }
  PosPinInSp[GndPin-1] = -1;   /* 9(=Pin 10) bzw. 11(=Pin 12) : Gnd-Pin */
  InvPinInSp[GndPin-1] = -1;
  if(! SynBit) {
    PosPinInSp[OEPin-1] = -1;      /* OE nicht in Matrix */
    InvPinInSp[OEPin-1] = -1;
  }
  for(i=0;i<AUSGAENGE/2;i++) {      /* die acht OLMCs */
    if((!i)?(! SynBit):(AC0Bit)) {  /* Rckkopplung ber eigene OLMC */
      /*---------- die ersten 4 OLMCs ---------------------*/
      if(Ausgaenge[i].LowAktiv) {
        if(Ausgaenge[i].Reg )    {
          InvPinInSp[LetztEingPin+i] = SpOLMC0 - (4 * i);   /* 11 bis 14*/
          PosPinInSp[LetztEingPin+i] = SpOLMC0 + 1 - (4 * i);
        }
        else {
          InvPinInSp[LetztEingPin+i] = SpOLMC0 - (4 * i);   /* 11 bis 14*/
          PosPinInSp[LetztEingPin+i] = SpOLMC0 + 1 - (4 * i);
        }
      }
      else {   /* high - aktiv */
        if(Ausgaenge[i].Reg )   {
          PosPinInSp[LetztEingPin+i] = SpOLMC0 - (4 * i);   /* 11 bis 14*/
          InvPinInSp[LetztEingPin+i] = SpOLMC0 + 1 - (4 * i);
        }
        else {
          PosPinInSp[LetztEingPin+i] = SpOLMC0 - (4 * i);   /* 11 bis 14*/
          InvPinInSp[LetztEingPin+i] = SpOLMC0 + 1 - (4 * i);
        }
      }
      /*---------- die letzten 4 OLMCs ---------------------*/
      if(Ausgaenge[7-i].LowAktiv) {
        if(Ausgaenge[7-i].Reg )    {
          InvPinInSp[LetztEingPin+7-i] = SpOLMC7 + (4 * i);  /* 18 bis 15*/
          PosPinInSp[LetztEingPin+7-i] = SpOLMC7 + 1 + (4 * i);
        }
        else {
          InvPinInSp[LetztEingPin+7-i] = SpOLMC7 + (4 * i);  /* 18 bis 15*/
          PosPinInSp[LetztEingPin+7-i] = SpOLMC7 + 1 + (4 * i);
        }
      }
      else {
        if(Ausgaenge[7-i].Reg )    {
          PosPinInSp[LetztEingPin+7-i] = SpOLMC7 + (4 * i);  /* 18 bis 15*/
          InvPinInSp[LetztEingPin+7-i] = SpOLMC7 + 1 + (4 * i);
        }
        else {
          PosPinInSp[LetztEingPin+7-i] = SpOLMC7 + (4 * i);  /* 18 bis 15*/
          InvPinInSp[LetztEingPin+7-i] = SpOLMC7 + 1 + (4 * i);
        }
      }
    }
    else   {      /* Rckkopplung nur ber untergeordnete OLMC */
      /*------- die ersten vier OLMCs -----------------*/
      PosPinInSp[LetztEingPin+i] = -1; /* Lschen, falls innere*/
      InvPinInSp[LetztEingPin+i] = -1; /* OLMC oder AC1(m)=0 */
      if ( (i<3) &&      /* innere OLMCs nicht rckfhrbarbar */
           (!AC0Bit) ) {  /* Rckkopplung nur bei AC0==0 */
        if(AC1Bits[i]) {       /* AC1(n)-Bit bercksichtigen */
          if(Ausgaenge[i].LowAktiv) {
            InvPinInSp[LetztEingPin+i] = SpOLMC0 - (4 * (i+1));
            PosPinInSp[LetztEingPin+i] = SpOLMC0 + 1 - (4 * (i+1));
          }
          else {
            PosPinInSp[LetztEingPin+i] = SpOLMC0 - (4 * (i+1));
            InvPinInSp[LetztEingPin+i] = SpOLMC0 + 1 - (4 * (i+1));
          }
        }
      }
      /*------- die letzten vier OLMCs -----------------*/
      PosPinInSp[LetztEingPin+7-i] = -1; /* Lschen, falls innere*/
      InvPinInSp[LetztEingPin+7-i] = -1; /* OLMC oder AC1(m)=0 */
      if( (i<3) &&       /* innere OLMCs nicht rckfhrbarbar */
          (!AC0Bit) ) {  /* Rckkopplung nur bei AC0==0 */
        if(AC1Bits[AUSGAENGE-1-i]) {       /* AC1(m)-Bit bercksichtigen */
          if(Ausgaenge[AUSGAENGE-1-i].LowAktiv) {
            InvPinInSp[LetztEingPin+AUSGAENGE-1-i] = SpOLMC7 +  (4 * (i+1));
            PosPinInSp[LetztEingPin+AUSGAENGE-1-i] = SpOLMC7 + 1 + (4 * (i+1));
          }
          else {
            PosPinInSp[LetztEingPin+AUSGAENGE-1-i] = SpOLMC7 +  (4 * (i+1));
            InvPinInSp[LetztEingPin+AUSGAENGE-1-i] = SpOLMC7 + 1 + (4 * (i+1));
          }
        }
      }  /* if(i<3) */
    }  /* Rckfhrung ber untergeordnete OLMCs */
  }   /* for(i) */
  switch(GalTyp) {
    case GAL16V8NR: {
          if(SynBit) {
            PosPinInSp[OEPin-1] = SpOLMC0;    /* 10 = Pin 11 : OE-Pin */
            InvPinInSp[OEPin-1] = SpOLMC0 + 1;
          }
          for(i=(LetztEingPin+AUSGAENGE);i<MAX_PIN;i++) { /* 19 bis 23 */
            PosPinInSp[i] = -1;            /* unbenutzte Pins und Vcc-Pin */
            InvPinInSp[i] = -1;            /* lschen */
          }
    } break;
    case GAL20V8NR: {
          if(! SynBit) {  /* Pin 14 und 23 werden auf Matrix gefhrt */
            PosPinInSp[13] = 38;
            InvPinInSp[13] = 39;
            PosPinInSp[22] = 2;
            InvPinInSp[22] = 3;
          }
          else {  /* OE direkt auf Matrix */
            PosPinInSp[OEPin-1] = SpOLMC0 + 4;    /* 10 = Pin 11 : OE-Pin */
            InvPinInSp[OEPin-1] = SpOLMC0 + 5;
            PosPinInSp[13] = 34;          /* Pin 14 und 23 ber OLMCs */
            InvPinInSp[13] = 35;          /* auf Matrix gefhrt */
            PosPinInSp[22] = 6;
            InvPinInSp[22] = 7;
          }
          PosPinInSp[VccPin-1] = -1;    /* Vcc-Pin sperren */
          InvPinInSp[VccPin-1] = -1;
    } break;
  }
  return(KEIN_FEHLER);
}


