/******************************************************************************
 *  Datei Protokol.C
 *  Enthlt die Funktion Protokoll() fr das Programm GalAsm, den Logic-
 *  Compiler / -Assembler zur Erstellung einer JEDEC-Datei
 *  fr die GAL-Programmierung.
 *
 *  Version               1.2
 *  Erstellt am           16.08.1990
 *  Letzte nderung am    26.05.1991
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien: GalAsm.H      Makros und Def. der Var.'typen
 *                           GalGlVar.H    Deklaration der globalen Variablen
 *
 *  Bentigte C-Module:      GalGlVar.C    Definition der globalen Variablen
 *
 *****************************************************************************/

#include <stdio.h>

#include "galasm.h"
#include "galglvar.h"

/*--------------------------------------------------------------------------
 |  Funktion Protokoll()
 |  Schreibt alle internen Informationen und Daten in eine Textdatei.
 |  Fehlercodes:
 */
#define  PROTOKOLL_ERR   60000L    /* Basis-Fehlernummer von Protokoll() */
/*
 *--------------------------------------------------------------------------*/
long Protokoll(DatName)
char *DatName;
{
register int i,LfdPinNr;

  gotoxy(0,MSG_ZEILE);
  end_line();
#ifdef GALASM_DEBUG  
  printf("Anlegen der Protokoll-Datei\n");
#endif
  fprintf(ProtFP,
  "\t\t\tProtokolldatei des Logik-Compilers GalAsm, \n\t\t\t\tVer.%s (%s) von V.Bosch\n",
          VersionNr,ErstDatum);
  fprintf(ProtFP,"Bearbeitete Datei: %s\n",DatName);
  fprintf(ProtFP,"Ziel PLD         : ");
  switch(GalTyp) {
    case GAL16V8NR: {
        fprintf(ProtFP,"%s\n\n",GAL16V8NAME);
        } break;
    case GAL20V8NR: {
        fprintf(ProtFP,"%s\n\n",GAL20V8NAME);
        } break;
  }
  if(FixMaxon != 0) {
    fprintf(ProtFP,"\n Vorsicht: Assemblierung mit Fix fuer MAXon Galprog MGP Ver. 2.0 und A-GALs\n");
    fprintf(ProtFP,"\t -- SYN- und AC0-Bit vertauscht!\n\n");
  }
  fprintf(ProtFP,"Pinbelegung des PLDs:\n");
  fprintf(ProtFP,"====================\n");
  fprintf(ProtFP,"      %9s     __________  \n"," ");
  fprintf(ProtFP,"      %9s  --+   \\__/   +--\n"," ");
  for(LfdPinNr = 1; LfdPinNr <= AnzPins/2; LfdPinNr++) {
    if(LfdPinNr== GndPin) {
      fprintf(ProtFP,"    %-8s    |%2d|          |%2d| ","Gnd",
           LfdPinNr,AnzPins+1-LfdPinNr);
    }
    else {
      for(i=0;i<EINGAENGE;i++) {
        if(Eingaenge[i].PinNummer == LfdPinNr) {
          if(Eingaenge[i].LowAktiv) fprintf(ProtFP,"   %c",EING_NEGATION);
          else fprintf(ProtFP,"    ");
          fprintf(ProtFP,"%-8s",Eingaenge[i].SigName);
          if(Eingaenge[i].Genutzt) fprintf(ProtFP," ->");
          else fprintf(ProtFP," X ");
          fprintf(ProtFP," |%2d|          |%2d| ",LfdPinNr,AnzPins+1-LfdPinNr);
        }
      }
    }
    if(AnzPins+1-LfdPinNr== VccPin) {
      fprintf(ProtFP,"      Vcc\n");
    }
    else {
      for(i=0;i<EINGAENGE;i++) {
        if(Eingaenge[i].PinNummer == AnzPins+1-LfdPinNr) {
          if(Eingaenge[i].Genutzt) fprintf(ProtFP," <-");
          else fprintf(ProtFP,"  X");
          if(Eingaenge[i].LowAktiv) fprintf(ProtFP,"  %c",EING_NEGATION);
          else fprintf(ProtFP,"   ");
          fprintf(ProtFP,"%-8s\n",Eingaenge[i].SigName);
        }
      }
      for(i=0;i<AUSGAENGE;i++) {
        if(Ausgaenge[i].PinNummer == AnzPins+1-LfdPinNr) {
          if(Ausgaenge[i].Reg) {   /* kein Eingang ! */
            fprintf(ProtFP," =:> ");
          }
          else {
            if((Ausgaenge[i].Hochohmig)   &&
               (!(Ausgaenge[i].Rueckkop))    ) {
              fprintf(ProtFP,"  X  ");
            }
            else {
              if((Ausgaenge[i].Hochohmig)                               ||
                 ((Ausgaenge[i].OEFreig)   && (Ausgaenge[i].Rueckkop))  ||
                 ((Ausgaenge[i].ProdFreig) && (Ausgaenge[i].Rueckkop))    )
                    fprintf(ProtFP," <-");
              else fprintf(ProtFP,"  -");
              if(Ausgaenge[i].Ausg) {   /* kein Eingang ! */
                fprintf(ProtFP,"> ");
              }
              else
                fprintf(ProtFP,"  ");
            }
          }
          if(Ausgaenge[i].LowAktiv) fprintf(ProtFP,"%c",EING_NEGATION);
          else fprintf(ProtFP," ");
          fprintf(ProtFP,"%-8s\n",Ausgaenge[i].SigName);
        }
      }
    }
    if(LfdPinNr < AnzPins/2)
      fprintf(ProtFP,"      %9s  ==+          +==\n"," ");
  }
  fprintf(ProtFP,"       %9s --+__________+--\n\n"," ");
  fprintf(ProtFP,"Legende:  ->: kombinatorischer Aus- bzw Eingang\n");
  fprintf(ProtFP,"         =:>: sequentieller (registered) Ausgang\n");
  fprintf(ProtFP,"         <->: kombinator. Aus- und Eingang (Tristate)\n");
  fprintf(ProtFP,"          X : nicht benutzt, sollte auf Gnd gelegt werden");
  fprintf(ProtFP,"\n");
  /*------ Architekturkontrollwort -----------------------------------*/
  fprintf(ProtFP,"\nArchitekturkontrollbits:\n");
  fprintf(ProtFP,"=======================\n");
  fprintf(ProtFP," %8s  Syn   = %d\n"," ",SynBit);
  fprintf(ProtFP," %8s  AC0   = %d\n"," ",AC0Bit);
  for(i=AUSGAENGE-1;i>=0;i--) {
    fprintf(ProtFP," %-8s: AC1(%d)=%d, XOR(%d)=%d\n",
               Ausgaenge[i].SigName,
               AUSGAENGE-i,AC1Bits[i],
               AUSGAENGE-i,!(Ausgaenge[i].Negieren));
  }
  /*-------------- Konfiguration der OLMCs -------------------------*/
  fprintf(ProtFP,"\nKonfiguration der OLMCs:\n");
  fprintf(ProtFP,"=======================\n");
  for(i=AUSGAENGE-1;i>=0;i--) {
    fprintf(ProtFP,"OLMC %d (Pin %2d): ",AUSGAENGE-i,
       Ausgaenge[i].PinNummer);
    if(Ausgaenge[i].LowAktiv) fprintf(ProtFP,"%c",EING_NEGATION);
    else fprintf(ProtFP," ");
    fprintf(ProtFP,"%-9s",Ausgaenge[i].SigName);
    if(!(Ausgaenge[i].Ausg)) {
      if(Ausgaenge[i].Rueckkop) {
        fprintf(ProtFP,"Eing.\n");
      }
      else {
        fprintf(ProtFP,"nicht benutzt\n");
      }
    }
    else {   /* 1. else */
      if(Ausgaenge[i].Reg) {
        fprintf(ProtFP,"seq.Ausg.,Freig.:/%s",
           Eingaenge[OEPin-2].SigName);
        if(Ausgaenge[i].Rueckkop) {
          fprintf(ProtFP,", rckgek.\n");
        }
        else fprintf(ProtFP,"\n");
      }
      else {   /* 2. else */
        if( (Ausgaenge[i].OEFreig)   ||
           ((Ausgaenge[i].ProdFreig) && (!(Ausgaenge[i].Hochohmig))) ||
           ((Ausgaenge[i].ProdFreig) && (!(Ausgaenge[i].Niederohmig))) ) {
          fprintf(ProtFP,"Trist. Aus");
          if(Ausgaenge[i].Rueckkop)  {
            fprintf(ProtFP,"-/Eing., rckgek.");
          }
          else {
            fprintf(ProtFP,"g.");
          }
          if(Ausgaenge[i].OEFreig)  {
            fprintf(ProtFP,", Freig.:/%s",Eingaenge[OEPin-2].SigName);
          }
          else {
            if(((Ausgaenge[i].ProdFreig) && (!(Ausgaenge[i].Hochohmig))) ||
               ((Ausgaenge[i].ProdFreig) && (!(Ausgaenge[i].Niederohmig))) ){
              fprintf(ProtFP,", Freig.:Prod'term");
            }
          }
          fprintf(ProtFP,"\n");
        }  /*    if( (Ausgaenge[i].OEFreig)   ||   */
        else {
          fprintf(ProtFP,"kombinat. Ausg.");
          if(Ausgaenge[i].Rueckkop)
            fprintf(ProtFP,", rckgek.\n");
          else fprintf(ProtFP,"\n");
        }
      }  /* 2. else */
    }     /* 1. else */
  }   /* for(i=AUSGAENGE-1;i>=0;i--) */
  return(KEIN_FEHLER);
}


